package errortest.app;


import java.io.FileWriter;
import java.io.IOException;

import org.ffd2.austenexample.ExampleGUI;
import org.ffd2.austenexample.ParseAction;
import org.ffd2.austenx.runtime.GeneralErrorCatcher;
import org.ffd2.austenx.runtime.ParserErrorCatcher;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleStringBuffer;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.io.IOTools;
import org.ffd2.solar.language.BasicErrorTracker;
import org.ffd2.solar.language.ErrorTracker;
import org.ffd2.solar.language.GeneralCommonErrorOutput;
import org.ffd2.solar.language.SpecificCommonErrorOutput;
import org.ffd2.solar.language.StringBasedCharacterSource;
import org.ffd2.solar.pretty.PrettyPrinter;
import org.ffd2.solar.pretty.SimpleStringBufferBasedPrettyPrinter;
import org.ffd2.solar.pretty.WriterBasedPrettyPrinter;

import errortest.austen.lexi.main.ErrorTestLexParser;
import errortest.austen.packrat.library.ErrorTestPackrat;
import errortest.austen.packrat.library.FileRootBuilderForErrorTest;
import errortest.austen.peg.base.FilePatternPeer;
import errortest.austen.peg.base.FunctionDefPatternPeer;

public class ErrorTest {
	private final static class FileRoot implements FileRootBuilderForErrorTest,FilePatternPeer {
		private final SimpleVector<FunctionBlock> functions_ = new SimpleVector<FunctionBlock>();
		private final GeneralCommonErrorOutput errorBase_;
		public FileRoot(GeneralCommonErrorOutput errorBase) {
			this.errorBase_ = errorBase;
			
		}
		public FilePatternPeer buildFile() { return this; }

		public void end() { 	}

		public void output(PrettyPrinter pp) {
			for(FunctionBlock function : functions_) {
				function.output(pp);
				
			}
		}

		public FunctionDefPatternPeer addFunctionDefForFunctionBlock(int keywordLine, int keywordCharacter, String name) {
			SpecificCommonErrorOutput functionError = this.errorBase_.createSpecificCommonErrorOutput(keywordLine, keywordCharacter);
			FunctionBlock function = new FunctionBlock(name,functionError);
			this.functions_.addElement(function);
			return function;
		}

		
	}
	public static final void doParse(String fileName, String outputFileName) throws IOException {
		String text = IOTools.readFileToStringLeavingNewLines(fileName);
		ErrorTestLexParser lexParser = new ErrorTestLexParser( );
		BasicErrorTracker errorTracker = new BasicErrorTracker();
		GeneralCommonErrorOutput errorBase = new GeneralCommonErrorOutput(fileName,errorTracker );
		ParserErrorCatcher errorCatcher = new GeneralErrorCatcher(errorBase );
		FileRoot target = new FileRoot(errorBase);
		ErrorTestPackrat.quickAndSimpleBuildFile(new StringBasedCharacterSource(text), lexParser.createSpecificParserForNormal(), target, errorCatcher );
		
		if(errorTracker.isHadError()) {
			System.out.println("Problem parsing file '"+fileName+"'");
			errorTracker.outputAllToSystemOut();
		} else {
			System.out.println("Parsed file '"+fileName+"'");
			FileWriter output = new FileWriter(outputFileName);
			try {
				System.out.println("Outputing");
				WriterBasedPrettyPrinter pp = new WriterBasedPrettyPrinter(output);
				target.output(pp);
				output.flush();
				System.out.println("Outputed");
			} finally {
				output.close();
			}
		}
		
	}
	
	private final static class ErrorParseAction implements ParseAction {

		public String doParse(String inputText, ErrorTracker errorTracker) {
			ErrorTestLexParser lexParser = new ErrorTestLexParser( );
			GeneralCommonErrorOutput errorBase = new GeneralCommonErrorOutput("Input",errorTracker );
			ParserErrorCatcher errorCatcher = new GeneralErrorCatcher(errorBase );
			FileRoot target = new FileRoot(errorBase);
			ErrorTestPackrat.quickAndSimpleBuildFile(new StringBasedCharacterSource(inputText), lexParser.createSpecificParserForNormal(), target, errorCatcher);
			
			//if(errorTracker.isHadError()) {
			// We don't care if we've had an error - we want to show what has been recovered.
				
			SimpleStringBufferBasedPrettyPrinter pp = new SimpleStringBufferBasedPrettyPrinter(new SimpleStringBuffer());
			target.output(pp);
			return pp.toString();
			//}
			
		}
		
	}
	public static final void main(String[] args) {
//		String inputFileName ="example.indent";
//		String outputFileName ="example.curly";
//		
//		try {
//			doParse(inputFileName,outputFileName);
//		} catch(IOException e) {
//			System.out.println("IO problems parsing file '"+inputFileName+"':"+e.getLocalizedMessage());
//		}
		ExampleGUI gui = new ExampleGUI("Error test", "function test(x) {\nx=4;\n}", new ErrorParseAction());
		gui.show();
	}
}
