package errortest.app;


import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.pretty.PrettyPrinter;
import errortest.austen.peg.base.*;
import errortest.austen.peg.base.ExpressionPeer.AdditionPatternPeer;
import errortest.austen.peg.base.ExpressionPeer.DivisionPatternPeer;
import errortest.austen.peg.base.ExpressionPeer.FunctionPatternPeer;
import errortest.austen.peg.base.ExpressionPeer.Indirect;
import errortest.austen.peg.base.ExpressionPeer.MultiplicationPatternPeer;
import errortest.austen.peg.base.ExpressionPeer.PrimitivePatternPeer;
import errortest.austen.peg.base.ExpressionPeer.SubtractionPatternPeer;

public class ExpressionBlock implements ExpressionPeer.Indirect {
	private Handler handler_;
	public ExpressionBlock() {
		
	}
	public void output(PrettyPrinter pp) {
		this.handler_.output(pp);
	}
	// --- AustenX bits 
	public AdditionPatternPeer createAddition() {
		BinaryHandler handler = new BinaryHandler("+");
		this.handler_ = handler;
		return handler;
	}
	public SubtractionPatternPeer createSubtraction() {
		BinaryHandler handler = new BinaryHandler("-");
		this.handler_ = handler;
		return handler;
	}
	public MultiplicationPatternPeer createMultiplication() {
		BinaryHandler handler = new BinaryHandler("*");
		this.handler_ = handler;
		return handler;
	}
	public DivisionPatternPeer createDivision() {
		BinaryHandler handler = new BinaryHandler("/");
		this.handler_ = handler;
		return handler;
	}
	
	public FunctionPatternPeer createFunction(String name) {
		FunctionCallHandler callHandler = new FunctionCallHandler(name);
		this.handler_ = callHandler;
		return callHandler;
	}

	public void createVariable(final String name) {
		this.handler_ = new Handler() {
			public void output(PrettyPrinter pp) { pp.print(name); }
		};
	}

	public PrimitivePatternPeer createPrimitive() {
		return new PrimitivePatternPeer() {

			public void end() {}

			public void addNumber(final int value) {
				handler_ = new Handler() { 
					public void output(PrettyPrinter pp) { pp.pint(value); }
				};
			}

			public void addBool(final boolean value) {
				handler_ = new Handler() { 
					public void output(PrettyPrinter pp) { pp.pboolean(value); }
				};
			}

			public void addString(final String value) {
				handler_ = new Handler() { 
					public void output(PrettyPrinter pp) { pp.pchar('"').print(value).pchar('"'); }
				};
			}
			
		};
	}
	// --------------------------------------------------------------------------
	private final static class BinaryHandler implements Handler,AdditionPatternPeer,SubtractionPatternPeer,DivisionPatternPeer,MultiplicationPatternPeer {
		private final String type_;
		private final ExpressionBlock left_;
		private final ExpressionBlock right_;
		public BinaryHandler(String type) {
			this.type_ = type;
			this.left_ = new ExpressionBlock();
			this.right_ = new ExpressionBlock();
			
		}
		public void end() { }
		public ExpressionPeer.Indirect getExpressionForLeft() { return this.left_; }
		public ExpressionPeer.Indirect getExpressionForRight() { return this.right_; }

		public void output(PrettyPrinter pp) {
			pp.print("( ");
			left_.output(pp);
			pp.space().print(this.type_).space();
			right_.output(pp);

			pp.print(" )");
		}
	}
	// --------------------------------------------------------------------------

	private final static class FunctionCallHandler implements Handler,FunctionPatternPeer {
		private final String functionName_;
		private final SimpleVector<ExpressionBlock> values_ = new SimpleVector<ExpressionBlock>();
		public FunctionCallHandler(String functionName) {
			this.functionName_ = functionName;
			
		}

		public void end() { 	
		}

		public ExpressionPeer.Indirect getExpressionForValue() {
			ExpressionBlock value = new ExpressionBlock();
			this.values_.addElement(value);
			return value;
		}
		public void output(PrettyPrinter pp) { 
			pp.print(this.functionName_).pchar('('); 
				boolean comma = false;
				for(ExpressionBlock value : this.values_) {
					if(comma) {
						pp.print(", ");
					} else {
						comma = true;
					}
					value.output(pp);
				}
			pp.pchar(')');
		}

	}
	// --------------------------------------------------------------------------
	private static interface Handler {
		public void output(PrettyPrinter pp);
	}

}
