package errortest.app;

import org.ffd2.austenx.runtime.BaseToken;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.SpecificCommonErrorOutput;
import org.ffd2.solar.pretty.PrettyPrinter;

import errortest.austen.peg.base.FunctionDefPatternPeer;
import errortest.austen.peg.base.StatementPeer;
import errortest.austen.peg.base.StatementPeer.Indirect;

public class FunctionBlock implements FunctionDefPatternPeer {
	private final SimpleVector<String> parameterNames_ = new SimpleVector<String>();
	private final StatementsBlock statements_;
	private final String functionName_;
	private final SpecificCommonErrorOutput error_;
	public FunctionBlock(String functionName, SpecificCommonErrorOutput error) {
		this.functionName_ = functionName;
		this.error_ = error;
		this.statements_ = new StatementsBlock(this.error_);
		
	}
	public void output(PrettyPrinter pp) {

		pp.print("function ").print(this.functionName_).print("(").print(this.parameterNames_.toString(", ")).print(") {").increaseIndent().newLine();
			this.statements_.output(pp);
		pp.decreaseIndent().print("}").newLine().newLine();
	}
	// --- AustenX bits
	public void end() { }
	public void addP(String value) { this.parameterNames_.addElement(value); }
	public StatementPeer.Indirect getStatementForStatements() { return this.statements_; }
	public void errorError(String errorHint, BaseToken currentMaximallyAccessedToken) {
		SpecificCommonErrorOutput newError = error_.createAdjusted(currentMaximallyAccessedToken.getLineNumber(), currentMaximallyAccessedToken.getCharacterNumber());
		newError.generalSyntaxError(errorHint!=null? errorHint : "unexpected token '"+currentMaximallyAccessedToken.getStringValue()+"'");
	}
	public void errorParameterError(String errorHint, BaseToken currentMaximallyAccessedToken) {
		SpecificCommonErrorOutput newError = error_.createAdjusted(currentMaximallyAccessedToken.getLineNumber(), currentMaximallyAccessedToken.getCharacterNumber());
		newError.generalSyntaxError(errorHint!=null? errorHint : "unexpected token in parameters '"+currentMaximallyAccessedToken.getStringValue()+"'");

	}
	public void markParameterError(int line, int character) {
		SpecificCommonErrorOutput newError = error_.createAdjusted(line , character );
		newError.generalSyntaxError("unexpected token in parameters");

	}
	public void markParameterError(BaseToken markToken) {
		SpecificCommonErrorOutput newError = error_.createAdjusted(markToken.getLineNumber(), markToken.getCharacterNumber());
		newError.generalSyntaxError("unexpected token in parameters '"+markToken.getStringValue()+"'");

	}
	public void markMissingRightRoundError(BaseToken markToken) {
		SpecificCommonErrorOutput newError = error_.createAdjusted(markToken.getLineNumber(), markToken.getCharacterNumber());
		newError.generalSyntaxError("missing ')'");
	}
	public void markError(BaseToken markToken) {
		SpecificCommonErrorOutput newError = error_.createAdjusted(markToken.getLineNumber(), markToken.getCharacterNumber());
		newError.generalSyntaxError("unexpected token '"+markToken.getStringValue()+"'");
	}


}
