package errortest.app;


import org.ffd2.austenx.runtime.BaseToken;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.SpecificCommonErrorOutput;
import org.ffd2.solar.pretty.PrettyPrinter;

import errortest.austen.peg.base.StatementPeer;
import errortest.austen.peg.base.StatementPeer.AssignmentPatternPeer;
import errortest.austen.peg.base.StatementPeer.BlockPatternPeer;
import errortest.austen.peg.base.StatementPeer.ExpressionPatternPeer;
import errortest.austen.peg.base.StatementPeer.IfPatternPeer;
import errortest.austen.peg.base.StatementPeer.Indirect;

public class StatementsBlock implements  StatementPeer.Indirect {
	private final SimpleVector<StatementHandler> statements_ = new SimpleVector<StatementHandler>();
	private final SpecificCommonErrorOutput error_;
	public StatementsBlock(SpecificCommonErrorOutput error) {
		this.error_ = error;
		
	}
	/**
	 * Output, but don't include a final new line
	 * @param pp
	 */
	public void output(PrettyPrinter pp) {
		//Don't include an end newline...
		for(StatementHandler statement : statements_ ) {
			statement.output(pp);
			pp.newLine();
		}
	}
	// ---- AustenX bits ----


	public AssignmentPatternPeer createAssignment(String variable) {
		AssignmentStatement statement = new AssignmentStatement(variable);
		this.statements_.addElement(statement);
		return statement;
	}

	public ExpressionPatternPeer createExpression() {
		ExpressionStatement statement = new ExpressionStatement();
		this.statements_.addElement(statement);
		return statement;
	}

	public IfPatternPeer createIf(int keywordLine, int keywordCharacter) {
		IfStatement statement = new IfStatement(this.error_);
		this.statements_.addElement(statement);
		return statement;
	}
	// --------------------------------------------------------------------------
	private final static class SubBlock implements StatementHandler,BlockPatternPeer {
		private final StatementsBlock subStatements_;
		public SubBlock(SpecificCommonErrorOutput baseError) {
			this.subStatements_ = new StatementsBlock(baseError);
		}
		public void output(PrettyPrinter pp) {
			pp.print("{").increaseIndent().newLine();
				this.subStatements_.output(pp);
			pp.decreaseIndent().print("}");
		}
		public void end() { }

		public Indirect getStatementForStatement() { return this.subStatements_;	}
	} //end of class SubBlock
	public BlockPatternPeer createBlock() {
		SubBlock sub = new SubBlock(this.error_);
		this.statements_.addElement(sub);
		return sub;
	}
//	public void createError(String detailsErrorHint, BaseToken detailsMaxToken) {
//		Debug.println("ERROR!");
//
//		SpecificCommonErrorOutput newError = error_.createAdjusted(detailsMaxToken.getLineNumber(), detailsMaxToken.getCharacterNumber());
//		newError.generalSyntaxError(detailsErrorHint!=null? detailsErrorHint : "unexpected token '"+detailsMaxToken.getStringValue()+"'");
//	
//	}
//	public void createError(String detailsErrorHint, BaseToken detailsMaxToken, int lastSemicolonLine, int lastSemicolonCharacter) {
//		SpecificCommonErrorOutput newError = error_.createAdjusted(detailsMaxToken.getLineNumber(), detailsMaxToken.getCharacterNumber());
//		newError.generalSyntaxError(detailsErrorHint!=null? detailsErrorHint : "unexpected token '"+detailsMaxToken.getStringValue()+"'");
//	
//	}
	
	public void createError(BaseToken startTokenMarkToken, int lastSemicolonLine, int lastSemicolonCharacter) {
		SpecificCommonErrorOutput newError = error_.createAdjusted(startTokenMarkToken.getLineNumber(), startTokenMarkToken.getCharacterNumber());
		newError.generalSyntaxError("confustion starts at '"+startTokenMarkToken.getStringValue()+"'");
	}
	
}
