package indent.app;

import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.pretty.PrettyPrinter;

import indent.austen.peg.base.ExpressionPeer;
import indent.austen.peg.base.ExpressionPeer.FunctionPatternPeer;
import indent.austen.peg.base.ExpressionPeer.PrimitivePatternPeer;

public final class ExpressionBlock implements ExpressionPeer.Indirect {
	private Handler handler_;
	public ExpressionBlock() {
		
	}
	public void output(PrettyPrinter pp) {
		this.handler_.output(pp);
	}
	// --- AustenX bits 
	public FunctionPatternPeer createFunction(String name) {
		FunctionCallHandler callHandler = new FunctionCallHandler(name);
		this.handler_ = callHandler;
		return callHandler;
	}

	public void createVariable(final String name) {
		this.handler_ = new Handler() {
			public void output(PrettyPrinter pp) { pp.print(name); }
		};
	}

	public PrimitivePatternPeer createPrimitive() {
		return new PrimitivePatternPeer() {

			public void end() {}

			public void addNumber(final int value) {
				handler_ = new Handler() { 
					public void output(PrettyPrinter pp) { pp.pint(value); }
				};
			}

			public void addBool(final boolean value) {
				handler_ = new Handler() { 
					public void output(PrettyPrinter pp) { pp.pboolean(value); }
				};
			}

			public void addString(final String value) {
				handler_ = new Handler() { 
					public void output(PrettyPrinter pp) { pp.pchar('"').print(value).pchar('"'); }
				};
			}
			
		};
	}
	
	private final static class FunctionCallHandler implements Handler,FunctionPatternPeer {
		private final String functionName_;
		private final SimpleVector<ExpressionBlock> values_ = new SimpleVector<ExpressionBlock>();
		public FunctionCallHandler(String functionName) {
			this.functionName_ = functionName;
			
		}

		public void end() { 	
		}

		public ExpressionPeer.Indirect getExpressionForValue() {
			ExpressionBlock value = new ExpressionBlock();
			this.values_.addElement(value);
			return value;
		}
		public void output(PrettyPrinter pp) { 
			pp.print(this.functionName_).pchar('('); 
				boolean comma = false;
				for(ExpressionBlock value : this.values_) {
					if(comma) {
						pp.print(", ");
					} else {
						comma = true;
					}
					value.output(pp);
				}
			pp.pchar(')');
		}

	}
	// --------------------------------------------------------------------------
	private static interface Handler {
		public void output(PrettyPrinter pp);
	}
	
}