package indent.app;

import indent.austen.lexi.main.TestLexParser;
import indent.austen.packrat.library.FileRootBuilderForIndentTest;
import indent.austen.packrat.library.IndentTestPackrat;
import indent.austen.peg.base.ClassPatternPeer;
import indent.austen.peg.base.ExpressionPeer.Indirect;
import indent.austen.peg.base.FilePatternPeer;

import java.io.FileWriter;
import java.io.IOException;

import org.ffd2.austenexample.ExampleGUI;
import org.ffd2.austenexample.ParseAction;
import org.ffd2.austenx.runtime.GeneralErrorCatcher;
import org.ffd2.austenx.runtime.ParserErrorCatcher;
import org.ffd2.solar.general.SimpleStringBuffer;
import org.ffd2.solar.io.IOTools;
import org.ffd2.solar.language.BasicErrorTracker;
import org.ffd2.solar.language.ErrorTracker;
import org.ffd2.solar.language.GeneralCommonErrorOutput;
import org.ffd2.solar.language.StringBasedCharacterSource;
import org.ffd2.solar.pretty.PrettyPrinter;
import org.ffd2.solar.pretty.SimpleStringBufferBasedPrettyPrinter;
import org.ffd2.solar.pretty.WriterBasedPrettyPrinter;

public class IndentTest {
	private final static class FileRoot implements FileRootBuilderForIndentTest,FilePatternPeer {
		private ClassBlock lastClass_ = null;
		
		public FilePatternPeer buildFile() { return this; }

		public void end() { 	}

		public ClassPatternPeer addClassForClassBlock(int keywordLine, int keywordCharacter, String name) {
			this.lastClass_ = new ClassBlock(name,this.lastClass_);
			
			return this.lastClass_;
		}
		public void output(PrettyPrinter pp) {
			if(this.lastClass_!=null) {
				this.lastClass_.output(pp);
			}
		}
		
	}
	public static final void doParse(String fileName, String outputFileName) throws IOException {
		String text = IOTools.readFileToStringLeavingNewLines(fileName);
		IndentTracker trackerValue = new IndentTracker(100,2);
		TestLexParser lexParser = new TestLexParser(trackerValue );
		BasicErrorTracker errorTracker = new BasicErrorTracker();
		GeneralCommonErrorOutput errorBase = new GeneralCommonErrorOutput(fileName,errorTracker );
		ParserErrorCatcher errorCatcher = new GeneralErrorCatcher(errorBase );
		FileRoot target = new FileRoot();
		IndentTestPackrat.quickAndSimpleBuildFile(new StringBasedCharacterSource(text), lexParser.createSpecificParserForNormal(), target, errorCatcher , trackerValue);
		
		if(errorTracker.isHadError()) {
			System.out.println("Problem parsing file '"+fileName+"'");
			errorTracker.outputAllToSystemOut();
		} else {
			System.out.println("Parsed file '"+fileName+"'");
			FileWriter output = new FileWriter(outputFileName);
			try {
				System.out.println("Outputing");
				WriterBasedPrettyPrinter pp = new WriterBasedPrettyPrinter(output);
				target.output(pp);
				output.flush();
				System.out.println("Outputed");
			} finally {
				output.close();
			}
		}
		
	}
	
	private final static class IndentParseAction implements ParseAction {

		public String doParse(String inputText, ErrorTracker errorTracker) {
			IndentTracker trackerValue = new IndentTracker(100,2);
			TestLexParser lexParser = new TestLexParser(trackerValue );
			GeneralCommonErrorOutput errorBase = new GeneralCommonErrorOutput("Input",errorTracker );
			ParserErrorCatcher errorCatcher = new GeneralErrorCatcher(errorBase );
			FileRoot target = new FileRoot();
			IndentTestPackrat.quickAndSimpleBuildFile(new StringBasedCharacterSource(inputText), lexParser.createSpecificParserForNormal(), target, errorCatcher , trackerValue);
			
			if(errorTracker.isHadError()) {
				return null;
			} else {
				SimpleStringBufferBasedPrettyPrinter pp = new SimpleStringBufferBasedPrettyPrinter(new SimpleStringBuffer());
				target.output(pp);
				return pp.toString();
			}
			
		}
		
	}
	public static final void main(String[] args) {
//		String inputFileName ="example.indent";
//		String outputFileName ="example.curly";
//		
//		try {
//			doParse(inputFileName,outputFileName);
//		} catch(IOException e) {
//			System.out.println("IO problems parsing file '"+inputFileName+"':"+e.getLocalizedMessage());
//		}
		ExampleGUI gui = new ExampleGUI("Indent test", "", new IndentParseAction());
		gui.show();
	}
}
